﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("CheckIfGAShouldBeApplied", "Description missing")]
  [StorableClass]
  public class XCSCheckIfGAShouldBeApplied : SingleSuccessorOperator {

    #region Parameter Properties
    public ILookupParameter<IntValue> ThetaGAParameter {
      get { return (ILookupParameter<IntValue>)Parameters["ThetaGA"]; }
    }
    public ILookupParameter<IntValue> IterationParameter {
      get { return (ILookupParameter<IntValue>)Parameters["Iteration"]; }
    }
    public ILookupParameter<ItemArray<IntValue>> TimeStampsParameter {
      get { return (ILookupParameter<ItemArray<IntValue>>)Parameters["TimeStamps"]; }
    }
    public ILookupParameter<ItemArray<IntValue>> NumerositiesParameter {
      get { return (ILookupParameter<ItemArray<IntValue>>)Parameters["Numerosities"]; }
    }
    public IValueLookupParameter<BoolValue> RunGAParameter {
      get { return (IValueLookupParameter<BoolValue>)Parameters["RunGA"]; }
    }
    #endregion

    [StorableConstructor]
    protected XCSCheckIfGAShouldBeApplied(bool deserializing) : base(deserializing) { }
    protected XCSCheckIfGAShouldBeApplied(XCSCheckIfGAShouldBeApplied original, Cloner cloner)
      : base(original, cloner) {
    }
    public XCSCheckIfGAShouldBeApplied()
      : base() {
      Parameters.Add(new LookupParameter<IntValue>("ThetaGA"));
      Parameters.Add(new LookupParameter<IntValue>("Iteration"));
      Parameters.Add(new ScopeTreeLookupParameter<IntValue>("TimeStamps"));
      Parameters.Add(new ScopeTreeLookupParameter<IntValue>("Numerosities"));
      Parameters.Add(new ValueLookupParameter<BoolValue>("RunGA"));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new XCSCheckIfGAShouldBeApplied(this, cloner);
    }

    public override IOperation Apply() {
      int numerositySum = 0;
      int timestamNumerositySum = 0;

      var timestamps = TimeStampsParameter.ActualValue;
      var numerosities = NumerositiesParameter.ActualValue;
      int actualtime = IterationParameter.ActualValue.Value;
      int thetaGA = ThetaGAParameter.ActualValue.Value;

      if (timestamps.Length != numerosities.Length) {
        throw new ArgumentException("The number of timestamps and numerosities is not the same.");
      }

      for (int i = 0; i < numerosities.Length; i++) {
        numerositySum += numerosities[i].Value;
        timestamNumerositySum += timestamps[i].Value * numerosities[i].Value;
      }

      RunGAParameter.ActualValue = new BoolValue(actualtime - (timestamNumerositySum / numerositySum) > thetaGA);

      return base.Apply();
    }
  }
}
